<?php
/*
Plugin Name: LinkWorth
Plugin URI: http://www.linkworth.com
Description: LinkWorth Easy Link Syndication for WordPress and WPMU
Author: LinkWorth
Version: 3.1.2
Author URI: http://www.linkworth.com/
License: GPLv2 or later
*/

/* Copyright 2008 LinkWorth (support@linkworth.com)
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

//PLUGINS ARE NOT INITIALIZED IN GLOBAL SPACE, SO WE MUST DEFINE GLOBALS AS SUCH.
global $lw_linkads;
$lw_linkads = array();

class LinkWorth
{
	//INTITIALIZE THE PLUGIN. DEFINE VARIABLES, ADD ACTIONS, OPTIONALLY INSTALL THE PLUGIN.
	//ALSO, LOAD LW DATA.
	function init()
	{
		global $lw_linkads,$wp_rewrite,$lw_options,$lw_vercurrent;
		$lw_vercurrent = '319';

		add_action('admin_menu', array(&$this,'admin_menu_item'));
		//ADD DEBUGGING INFORMATION.
		add_action('wp_footer', array(&$this,'debug'));

		//IF THE OPTIONS ARRAY DOESN'T EXIST, THIS IS THE FIRST RUN.
		if(!is_array(get_option('lw_options')))
		{
			$this->install();
		}
		else
		{
			$this->upgrade();
		}

		$lw_options = get_option('lw_options');
		$this->set_constants();
		$this->get_ads($lw_update_ads = false);

		if(!$lw_options['billboard_base'])
		{
			$lw_options['billboard_base'] = 'linkworth';
		}

		//IF WE HAVE ADS.
		if(is_array($lw_linkads))
		{
			$this->setup_filters_and_actions();
		}
		else
		{
			//DELETE ANY OLD ADS.
			$this->lw_linkintxt_dbcleanup();
			//ADD DEBUG INFORMATION.
			$this->debug('No Linkworth ads found.');
			$lw_linkads = array();
		}

		//ADD LINKWORDS TO FOOTER
		add_action('wp_footer', array(&$this,'footer'));

		if(!isset($_GET['lw_update_now']) && is_array($lw_linkads))
		{
			//UPDATE THE BILLBOARD ADS. THESE ARE USED WITH RE-WRITES, SO IT IS BEST TO HAVE IT ALL READY.
			$this->billboards_update();
			//UPDATE THE OLD LINKS
			$this->linkintxts_update();
		}

		//DISABLES SILENT RUNNING WHEN SPECIFIED.
		if($_GET['silent'] == 'no')
		{
			$lw_options['disable_silent'] = 1;
		}

		add_action('admin_menu', array($this,'admin_init'));

		//SAVING CHANGES
		add_filter('edit_post',    array($this, 'update_post_options'));
		add_filter('publish_post', array($this, 'update_post_options'));
		add_filter('save_post',    array($this, 'update_post_options'));
	}

	function admin_init()
	{
		add_meta_box( 'linkworth_enable', 'Linkworth Options', array($this,'post_options'), 'post', 'side');
		add_meta_box( 'linkworth_enable', 'Linkworth Options',  array($this,'post_options'), 'page', 'side' );
	}

	//JUST ADD THE FILTERS AND ACTIONS.
	function setup_filters_and_actions()
	{
		global $wp_rewrite;
			//AUTOMATIC FILTER
			//ADD BILLBOARD ADS AFTER CONTENT.
			add_filter('the_content', array(&$this,'after_content'),12);
			//MANUAL THEME EDITOR
			//ADD "TYPE 1" ADS
			add_action('lw_after_content', array(&$this,'lw_after_content'));

			// WRAPS POST CONTENT IN DIV TAGS.
			add_filter('the_content', array(&$this,'linkwords_filter'),11);
			// LINKINTXT FILTER ON POST CONTENT
			add_filter('the_content', array(&$this,'lw_contentreplace'));
	
			//AUTOMATIC FILTER
			//ADD "TYPE 1" ADS
			add_filter('loop_end', array(&$this,'after_loop'));
			//MANUAL THEME EDIT
			//ADD "TYPE 1" ADS
			add_action('lw_after_loop', array(&$this,'lw_after_loop'));
			//BILLBOARDS
			add_filter('query_vars', array($this,'addQueryVar'));
			//BILLBOARDS
			add_action('parse_query', array($this,'Query'));
	}

	//DEFINE THE CONSTANTS IF THEY ARE NOT ALREADY SET.
	//DEFINE THE OPTIONS IF THEY ARE NOT ALREADY SET.
	function set_constants()
	{
		global $lw_options;

		//IF THE CONSTANTS ARE NOT DEFINED, DEFINE THEM.
		if(!defined('LW_SITE_ID') || !defined('LW_SITE_HASH'))
		{
			if($lw_options['site_id'] && $lw_options['site_hash'])
			{
				define('LW_SITE_ID', $lw_options['site_id']);
				define('LW_SITE_HASH', $lw_options['site_hash']);;
			}
			else
			{
				define('LW_SITE_ID', '');
				define('LW_SITE_HASH', '');
			}
		}
		else
		{
			//IF THE OPTIONS ARE NOT DEFINED, DEFINE THEM.
			if(!$lw_options['site_id'] || !$lw_options['site_hash'])
			{
				$lw_options['site_id'] = LW_SITE_ID;
				$lw_options['site_hash'] = LW_SITE_HASH;
			}
		}

		//IF THE CONSTANTS ARE NOT DEFINED, DEFINE THEM.
		if(!defined('LW_WEBSITE_ID') || !defined('LW_HASH'))
		{
			if($lw_options['website_id'] && $lw_options['website_hash'])
			{
				define('LW_WEBSITE_ID', $lw_options['website_id']);
				define('LW_HASH', $lw_options['website_hash']);
			}
			else
			{
				define('LW_WEBSITE_ID', '');
				define('LW_HASH', '');
			}
		}
		else
		{
			//IF THE OPTIONS ARE NOT DEFINED, DEFINE THEM.
			if(!$lw_options['website_id'] || !$lw_options['website_hash'])
			{
				$lw_options['website_id'] = LW_WEBSITE_ID;
				$lw_options['website_hash'] = LW_HASH;
			}
		}
	}

	//INSTALL THE PLUGIN.
	function install()
	{
		global $wp_rewrite,$lw_vercurrent;

		//STORE AN ARRAY OF VALUES INTO THE DATABASE.
		$options['lw_timeout'] = 5;
		$options['lw_linkcolor'] = 1;
		$options['lw_version'] = $lw_vercurrent;
	
		//FLUSH THE REWRITE RULES.
		$wp_rewrite->flush_rules();
		add_option('lw_options',$options,'The main array of options for LinkWorth');
		add_option('lw_widget');
		add_option('lw_cache', '', 'Cache of the lw ads in pretty serialized form.');
		add_option('lw_cache_time', '', 'Time the cache was last checked.');
	}

	function upgrade()
	{
		global $lw_vercurrent,$wp_rewrite;

		$lw_version = get_option('linkworth_version');

		if($lw_version == $lw_vercurrent)
		{
			return;
		}

		if($lw_version < 300)
		{
			if(!$wp_rewrite->using_mod_rewrite_permalinks())
			{
				$this->save_mod_rewrite_rules();
			}
		}

		if($lw_version < 310)
		{
			$this->delete_sidebar_links();
		}

		update_option('linkworth_version',$lw_vercurrent);
	}

	//CHECK HOW OLD THE CACHE IS, IF IT ISN'T OLD MOVE ON.
	//IF THERE ISN'T A CACHE OR IT IS EXPIRED, DOWNLOAD THE NEW ADS AND PARSE THEM.
	function get_ads($lw_update_ads)
	{
		global $lw_linkads, $lw_testing;
		$lw_linkads = get_option('lw_cache');
		$last = get_option('lw_cache_time');

		if(!is_array($lw_linkads)  || (time() - $last) > 3600 || $lw_update_ads)
		{ 
			ini_set('default_socket_timeout', 10);

			$lw_advstring = $this->get_contents('http://www.linkworth.com/act/partner/code/plugin_feed.php?web_id=' . LW_WEBSITE_ID . '&hash=' . LW_HASH . '&format=xml');

			if($lw_advstring)
			{ 
				$lw_advstring = str_replace('&amp;','&',$lw_advstring);
				$lw_advstring = str_replace('&amp;','&',$lw_advstring);
				$lw_advstring = str_replace('&amp;','&',$lw_advstring);
				$lw_advstring = str_replace('&','&amp;',$lw_advstring);

				include_once('lw_includes/LinkWorth_parser.php');

				$xml_parser = & new LW_XML();
				$lw_linkads = $xml_parser->parse($lw_advstring);
				//RESYNC ARRAY TO START WITH THE AD BLOCKS;
				$lw_linkads = $lw_linkads['linkworth']['ads'];

				update_option('lw_cache',$lw_linkads);
				update_option('lw_cache_time',time());
			}
			else
			{
				//DELAY THE UPDATE FOR 10 MINUTES INCASE OF SERVER ISSUES.
				update_option('lw_cache_time',time()-3000);
			}
		}
	}

	function clean_description($link)
	{
		if($link['description'])
		{
			$desc = $link['description'];
			$desc = trim($desc);
			$desc = htmlspecialchars($desc);

			return $desc;
		}
	}

	function get_contents($url)
	{
		//TRY THE MOST DIRECT WAY FIRST
		if(ini_get('allow_url_fopen'))
		{
			$string = @file_get_contents($url);
		}
		elseif(function_exists('curl_init'))
		{
			$curl_handle = curl_init();
			curl_setopt($curl_handle,CURLOPT_URL,$url);
			curl_setopt($curl_handle,CURLOPT_CONNECTTIMEOUT,2);
			curl_setopt($curl_handle,CURLOPT_RETURNTRANSFER,1);
			$string = curl_exec($curl_handle);
			curl_close($curl_handle);
		}

		return $string;
	}

	// ---------------------------------------------------------------------------------------
	// ---------------------------------- BILLBOARD FUNCTIONS --------------------------------
	// ---------------------------------------------------------------------------------------

	function detectBillboard($posts)
	{
		global $wp;
		global $wp_query;

		$billboard = trim(get_query_var('linkworth'),'/');
		//GET THE BILLBOARDS ARRAY.
		$lw_billboards = get_option('lw_billboards');

		//CHECK IF THE REQUESTED BILLBOARD EXISTS.
		if($billboard && $lw_billboards[$billboard])
		{
			if($lw_billboards[$billboard]['in_theme'] != 1)
			{
				$this->full_page_billboards();
			}

			//ADD THE FAKE POST
			$posts=NULL;
			$posts[]=$this->createPost();
		
			//TRICK WP_QUERY INTO THINKING THIS IS A PAGE (NECESSARY FOR WP_TITLE() AT LEAST)
			$wp_query->is_page = true;
			//NOT SURE IF THIS ONE IS NECESSARY BUT MIGHT AS WELL SET IT LIKE A TRUE PAGE
			$wp_query->is_singular = true;
			$wp_query->is_home = false;
			$wp_query->is_archive = false;
			$wp_query->is_category = false;
			//LONGER PERMALINK STRUCTURES MAY NOT MATCH THE FAKE POST SLUG AND CAUSE A 404 ERROR SO WE CATCH THE ERROR HERE
			unset($wp_query->query["error"]);
			$wp_query->query_vars["error"]="";
			$wp_query->is_404=false;
		}

		return $posts;
	}

	function createPost()
	{
		/**
		 * What we are going to do here, is create a fake post.  A post
		 * that doesn't actually exist. We're gonna fill it up with
		 * whatever values you want.  The content of the post will be
		 * the output from your plugin.
		 */

		//CREATE A FAKE POST.
		$post = new stdClass;
		$post->post_author = 1;
		$post->post_title = '';
		$post->post_content = $this->get_billboard();
		$post->ID = -1;
		$post->post_status = 'static';
		$post->comment_status = 'closed';
		$post->ping_status = 'closed';
		$post->comment_count = 0;
		$post->post_date = '13 Dec 1991 20:45:54 GMT';
		$post->post_date_gmt = '13 Dec 1991 20:45:54 GMT';

		return($post);
	}

	//IF THIS IS A BILLBOARD PAGE, FUNCTION OUTPUTS THE BILLBOARD PAGE AND STOPS THE REST OF WP.
	function full_page_billboards()
	{
		$billboard = trim(get_query_var('linkworth'),'/');
		//GET THE BILLBOARDS ARRAY.
		$lw_billboards = get_option('lw_billboards');

		//CHECK IF THE REQUESTED BILLBOARD EXISTS.
		if($billboard && $lw_billboards[$billboard])
		{
			echo $this->get_contents('http://www.linkworth.com/act/partner/code/linkbb.php?bb_id='.$lw_billboards[$billboard]['url'].'&web_id='.LW_WEBSITE_ID."&hash=".LW_HASH);
			die();
		}
	}

	function get_billboard($a=null)
	{
		$billboard = trim(get_query_var('linkworth'),'/');
		//GET THE BILLBOARDS ARRAY.
		$lw_billboards = get_option('lw_billboards');

		$content = $this->get_contents('http://www.linkworth.com/act/partner/code/linkbb.php?bb_id='.$lw_billboards[$billboard]['url'].'&web_id='.LW_WEBSITE_ID."&hash=".LW_HASH."&in_page=1");

		$content = substr($content, strpos($content,'<body>')+6, strpos($content,'</body>')-strpos($content,'<body>'));

		return $content;
	}

	//ADD THE MOD_REWRITE RULES TO WORDPRESS. - SO WE DON'T HAVE TO CREATE A FILE.
	function rewrite($wp_rewrite)
	{
		global $lw_options;

		$feed_rules = array($lw_options[billboard_base].'/(.+)?' => 'index.php?linkworth='. $wp_rewrite->preg_index(1));

		$wp_rewrite->rules = $feed_rules + $wp_rewrite->rules;
	}

	function addQueryVar($wpvar_array)
	{
		global $lw_options;

		$wpvar_array[] = 'linkworth';

		return($wpvar_array);
	}

	function AddRewriteRules()
	{
		global $lw_options;

		$home_root = parse_url(get_option('home'));
		if(isset($home_root['path']))
		{
			$home_root = trailingslashit($home_root['path']);
		}
		else
		{
			$home_root = '/';
		}

		$insert = <<<block

	<IfModule mod_rewrite.c>
	RewriteEngine On
	RewriteBase $home_root
	RewriteRule ^$lw_options[billboard_base]/(.+)$ index.php?linkworth=$1
	</IfModule>
block;

		return $insert;
	}

	function do_pretty_links()
	{
		global $wp_rewrite;

		if($wp_rewrite->using_mod_rewrite_permalinks())
		{
			return 1;
		}

		if(get_option('linkworth_rules_added'))
		{
			return 1;
		}

		return 0;
	}

	function save_mod_rewrite_rules()
	{
		update_option('linkworth_rules_added',0);
		$home = get_option('home');
		$siteurl = get_option('siteurl');

		if($home != '' && $home != $siteurl)
		{
			$wp_path_rel_to_home = str_replace($home, '', $siteurl); /* $siteurl - $home */
			$pos = strpos($_SERVER["SCRIPT_FILENAME"], $wp_path_rel_to_home);
			$home_path = substr($_SERVER["SCRIPT_FILENAME"], 0, $pos);
			$home_path = trailingslashit( $home_path );
		}
		else
		{
			$home_path = ABSPATH;
		}

		$htaccess_file = $home_path.'.htaccess';

		//IF THE FILE DOESN'T ALREADY EXISTS CHECK FOR WRITE ACCESS TO THE DIRECTORY AND WHETHER OF NOT WE HAVE SOME RULES.
		//ELSE CHECK FOR WRITE ACCESS TO THE FILE.
		if((!file_exists($htaccess_file) && is_writable($home_path) && $this->AddRewriteRules()) || is_writable($htaccess_file))
		{
			if(apache_mod_loaded('mod_rewrite', true))
			{
				$rules = explode( "\n", $this->AddRewriteRules() );
				include_once($home_path.'wp-admin/includes/misc.php');
				update_option('linkworth_rules_added',1);
				return insert_with_markers( $htaccess_file, 'LinkWorth', $rules );
			}
		}

		return false;
	}

	//BEFORE WE DO EANYTHING ELSE, STOP AND CHECK IF THIS IS A "BILLBOARD"
	function Query($query)
	{
		global $wp_rewrite;
		
		if(get_query_var('linkworth'))
		{
			add_filter('the_posts',array(&$this,'detectBillboard'));
		}
	}

	//LOOP THROUGH THE ADS AND UPDATE THE BILLBOARD OPTION WITH ANY NEW ONES.
	function billboards_update()
	{
		global $lw_linkads;
		$lw_billboards = array();

		if(!is_array($lw_linkads) || !is_array($lw_linkads['linkbb']) || !is_array($lw_linkads['linkbb']['ad']))
		{
				update_option('lw_billboards',array());
				return;
		}

		$billboards = $lw_linkads['linkbb']['ad'];

		if(!is_array($billboards[0]))
		{
			$an_ad = $billboards;
			$billboards = null;
			$billboards[0] = $an_ad;
		}

		if(is_array($billboards))
		{
			foreach ($billboards as $billboard)
			{
				$lw_billboards[$billboard['pagename']] = array('url'=>$billboard['bbid'],'in_theme'=>$billboard['in_theme']);
			} //WHILE BILLBOARD CREATION LOOP ENDS
		}

		update_option('lw_billboards',$lw_billboards);
	}

	//LOOP THROUGH THE ADS AND CHECK FOR NEW LINKINTXTS. IF THERE ARE NEW LINKS, ADD THEM INTO AN ARAY.
	function linkintxts_update()
	{
		global $lw_linkads,$lw_options;
		//NEW LINKS
		$linkintxs = array();

		if(!is_array($lw_linkads['linkintxt']) || !is_array($lw_linkads['linkintxt']['ad']))
		{
			update_option('lw_linkintxs',array());
			return;
		}

		$links = $lw_linkads['linkintxt']['ad'];

		if(is_array($lw_linkads))
		{
			reset($lw_linkads);
		}

		if(count($links) > 0)
		{
			foreach($links as $link)
			{
				unset($add_word, $rep_word,$lw_dispurl);

				$add_word = $link['anchor'];
				$desc = ' title="'.$this->clean_description($link).'" ';
				$protocol = ($link['use_https'])? 'https://' : 'http://';
				$rep_word = "<a href=\"".$protocol.$link['url']."\"$desc>".$link['anchor']."</a>";
				$lw_dispurl = $link['webpageurl'];

				if($add_word != "")
				{
					$linkintxs[] = array('find_word' => $add_word, 'replace_word' => $rep_word, 'disp_url' => $lw_dispurl);
				}
			}
		}

		ob_start(); echo 'LinkinTxts:'; print_r($linkintxs); $ads = ob_get_contents(); ob_end_clean(); $this->debug($ads);

		update_option('lw_linkintxs',$linkintxs);
	}

	// ---------------------------------------------------------------------------------------
	// ---------------------------------- VARIOUS UTILITY FUNCTIONS --------------------------
	// ---------------------------------------------------------------------------------------

	//WRAPS THE POST CONTENT IN DIV.
	function linkwords_filter($content)
	{
		global $id;
		$lw_options = get_option('lw_options');

		if($lw_options['lw_cssmod'] == 1)
		{
			$content = "<div id='lw_context_ads'>".$content."</div>\n";
			return $content;
		}

		return $content;
	}

	//PERFORMS VARIOUS TESTS ON THE THEME.
	function testtheme($test='')
	{
		static $site_html;

		//LOAD THE WEBSITE ONLY ONCE PER TEST RUN.
		if(!$site_html)
		{
			$site_html = $this->get_contents(get_option('siteurl').'?silent=no');
		}
	
		if($test == 'can_get_ads')
		{
			if(function_exists('curl_init') || ini_get('allow_url_fopen'))
			{
				return true;
			}
		}

		if($test == 'footer')
		{
			if(strpos($site_html,'<!-- LINKWORDS FOOTER CODE STARTS -->'))
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		if($test == 'loopexists')
		{
			if(strpos($site_html,'<!--linkworth loop ads start-->') !== false)
			{
				return true;
			}
		}

		if($test == 'loop')
		{
			$loop_count = preg_match_all('@\<\!--linkworth loop ads start--\>@',$site_html,$matches);
			if($matches[0][0])
				return count($loop_count);
			return 0;
		}

		//NO $test OR ALL TESTS FALSE.
		return false;
	}

	function cloudcss($lw_linkcolor)
	{
		$lw_cssfontsize = (rand()%11)+12;
		$lw_font_key = rand(1,3);

		$lw_color_array = array(
			1 => array(1 => "#00AEFF", 2 => "#594C9B", 3 => "#008C2E"),
			2 => array(1 => "#798975", 2 => "#81FF60", 3 => "#2C8B14"),
			3 => array(1 => "#737B87", 2 => "#77AFFF", 3 => "#14438B"),
			4 => array(1 => "#877673", 2 => "#FF7160", 3 => "#8B2014"),
			5 => array(1 => "#912DAE", 2 => "#53A244", 3 => "#DF723F"),
			6 => array(1 => "#FFFF40", 2 => "#C0C0FF", 3 => "#FF80FF"),
			7 => array(1 => "#606060", 2 => "#A8A8A8", 3 => "#E2E2E2"),
			8 => array(1 => "#FF0000", 2 => "#FF0000", 3 => "#FF0000")
		);

		if($lw_linkcolor < 1 || $lw_linkcolor > 7)
		{
			$lw_linkcolor = 8;
		}

		$css_color = $lw_color_array[$lw_linkcolor][$lw_font_key];
		$css_size = $lw_cssfontsize;

		return array($css_color, $css_size);
	}

	//ADDS FOOTER TO WP.
	function footer()
	{
		global $lw_options, $id;

		ob_start();

		//ADD LINKWORTH COMMENT
		if($lw_options['disable_silent'] == 1)
		{
?>
				<!-- LINKWORDS FOOTER CODE STARTS -->
<?php
		}
?>
			<style type='text/css'>
				.lw_cad_link:link
				{
					text-decoration: underline;
					border-bottom: 1px;
				}

				.lw_cad_link:visited
				{
					text-decoration: underline;
					border-bottom: 1px;
				}

				.lw_cad_link:hover
				{
					text-decoration: underline;
					border-bottom: 2px;
				}
				</style>
				<script type="text/javascript" src="http://www.linkwords.net/context-ads/context_ads.php?prt_website_id=<?php echo LW_WEBSITE_ID ?>"></script>
<?php
		//ADD LINKWORTH COMMENT
		if($lw_options['disable_silent'] == 1)
		{
?>
				<!-- LINKWORDS FOOTER CODE ENDS -->
<?php
		}

		$linkwords_script_content = ob_get_contents();
		ob_end_clean();

		if(get_post_meta($id,'_disable_linkworth',true))
		{
			$display_linkwords = false;
		}
		else
		{
			$display_linkwords = true;
		}

		if(($lw_options['lw_cssmod'] == 1 && !is_single()) || ($lw_options['lw_cssmod'] == 1 && is_single() && $display_linkwords))
		{
			echo $linkwords_script_content;
		}
	}

	//ADDS LW MENU ITEM TO WP DASHBOARD
	//CODE FOR THE LINKWORTH TAB ON THE OPTIONS MENU
	function admin_menu_item()
	{
		if(function_exists('add_options_page'))
		{
			add_options_page('LinkWorth Ads Syndication', 'LinkWorth', 8, __FILE__, 'lw_configpage');
		}
	}

	function post_options()
	{
		global $post;

		if($_GET['post'])
		{
			$disabled = get_post_meta($_GET['post'],'_disable_linkworth',true);
		}
	?>
		<input type="checkbox" <?php echo ($disabled) ? 'checked="checked"' : '';?> name="disable_linkworth" id="disable_linkworth" value="1"/><label for="disable_linkworth">Disable LinkWords on this post?</label>
<?php
	}

	function update_post_options()
	{
		if($_POST['action'] == 'autosave')
		{
			return;
		}

		$id = $_REQUEST['post_ID'];

		delete_post_meta($id,'_disable_linkworth');

		if($_POST['disable_linkworth'])
		{
			add_post_meta($id, '_disable_linkworth', $_POST['disable_linkworth']);
		}
	}

	//UPDATES ADMIN PANEL OPTIONS FROM POSTED DATA
	function update_options($postdata)
	{
		while(list($name, $value) = each($postdata))
		{
			if(get_magic_quotes_gpc())
			{
				$value = stripslashes($value);
			}

			$options[$name] =$value;
		}

		update_option('lw_options',$options);

		return $options;
	}

	// ---------------------------------------------------------------------------------------
	// -----------------ADS BEFORE LOOP AND AFTER CONTENT FILTER FUNCTIONS -------------------
	// ---------------------------------------------------------------------------------------

	//AUTOMATICALLY APPENDS TEXT LINKS AFTER POST CONTENT. RETURNS $CONTENT AND ANY $ADS
	function after_content($content)
	{
		global $lw_options;

		//IF THIS IS A NORMAL POST OR PAGE, ADD ADS.
		if(is_single() && !is_admin() && !is_feed() && !$lw_options['nocontentads'])
		{
			$content = $content.$this->lw_after_content('return');
		}

		return $content;
	}

	/**********************************
	* Returns or echos any new ads.
	* If $return is passed, this function 
	* was called by our filter.
	* Otherwsie, it was the do_action
	***********************************/

	function lw_after_content($return = 0)
	{
		global $lw_linkads,$lw_options;
		$count = 0;

		//IF THIS IS A NORMAL POST OR PAGE, ADD ADS.
		if(is_single())
		{
			//LOOP THE DIFFERENT AD TYPES.
			foreach($lw_linkads as $ad_type => $ads)
			{
				if($ad_type == 'linkintxt' || ($ad_type == 'linkads' && $lw_options['lw_sidebar'])|| ($ad_type == 'linkbb' && $lw_options['lw_sidebar']) || ($ad_type == 'linkmura' && $lw_options['lw_murasidebar'])) continue;

				if(!is_array($ads['ad'][0]))
				{
					$an_ad = $ads['ad'];
					$ads['ad'] = null;
					$ads['ad'][0] = $an_ad;
				}

				//LOOP THROUGH ALL THE ADS OF THE SAME TYPE.
				foreach($ads['ad'] as $ad)
				{
					//IF THIS AD SHOULD DISPLAY ON ALL PAGES OR SINGLE PAGES.
					if($ad['location'] == 3  || $ad['location'] == 2)
					{ 
						//RUN THE DISPLAY FUNCTION FOR THIS $AD_TYPE (FUNCTIONS ARE NAMED AFTER THE XML NAMES.)
						$ad_html .= $this->$ad_type($ad,$count);
						$count ++;
					}
				}
			}
		}

		if($return)
		{
			return $ad_html;
		}

		echo $ad_html;
	}

	//AUTOMATICALLY ADDS ADS AFTER THE LOOP.
	function after_loop($content)
	{
		global $lw_linkads,$lw_options;
		//THE NUMBER OF WORDPRESS LOOPS THAT HAVE BEEN CALLED SO FAR.
		static $loop=1;

		if(!is_home())
		{
			return;
		}

		if($lw_options['disable_silent'])
		{
?>
			<!--linkworth loop ads start-->
<?php
		}

		$count = 0;
		$loop_number = $lw_options['loop_number'];

		if(!$loop_number)
		{
			$loop_number = 1;
		}

		if($loop == $loop_number  && !$lw_options['noloopads'])
		{
			$this->lw_after_loop();
		}

		$loop++;
	}

	//ECHO ALL ADS THAT BELONG ON THE HOMEPAGE.
	function lw_after_loop()
	{
		global $lw_linkads,$lw_options;
		//THE NUMBER OF WORDPRESS LOOPS THAT HAVE BEEN CALLED SO FAR.
		static $loop=1;

		if(!is_home())
		{
			return;
		}

		//LOOP THE DIFFERENT AD TYPES.
		foreach($lw_linkads as $ad_type => $ads)
		{
			//IGNORE BILLBOARD ADS AND SIDEBAR ADS WHEN THE SIDEBAR IS ENABLED. - ALWAYS IGNORE LINKINTXT ADS.
			if($ad_type == 'linkintxt' || (($ad_type == 'linkbb' || $ad_type == 'linkads') && $lw_options['lw_sidebar']) || ($ad_type == 'linkmura' && $lw_options['lw_murasidebar'])) continue;

			if(!is_array($ads['ad'][0]))
			{
				$an_ad = $ads['ad'];
				$ads['ad'] = null;
				$ads['ad'][0] = $an_ad;
			}

			//LOOP THROUGH ALL THE ADS OF THE SAME TYPE.
			foreach($ads['ad'] as $ad)
			{
				//IF THIS AD SHOULD DISPLAY ON ALL PAGES OR THE HOME PAGE.
				if($ad['location'] == 3  || $ad['location'] == 1)
				{ 
					//RUN THE DISPLAY FUNCTION FOR THIS $AD_TYPE (FUNCTIONS ARE NAMED AFTER THE XML NAMES.)
					$ad_html .= $this->$ad_type($ad,$count);
					$count++;
				}
			}
		}

		echo $ad_html;
	}
	

	function lw_links_sidebar()
	{
		global $lw_linkads,$lw_options;
		$lw_path = get_option('siteurl');

		if(isset($lw_linkads))
		{
			reset($lw_linkads);
		}

		if(count($lw_linkads['linkads']) + count($lw_linkads['linkbb']) > 0 || ($lw_options['lw_murasidebar']) && (count($lw_linkads['linkads']) + count($lw_linkads['linkbb'])+ count($lw_linkads['linkmura'])) > 0)
		{
			foreach($lw_linkads as $ad_type => $ads)
			{
				if($ad_type != 'linkads' && $ad_type != 'linkbb' && !($ad_type == 'linkmura' && $lw_options['lw_murasidebar'])) continue;

				if(!is_array($ads['ad'][0]))
				{
					$ad = $ads['ad'];
					$ads['ad'] = null;
					$ads['ad'][0] = $ad;
				}

				//LOOP THROUGH ALL THE ADS OF THE SAME TYPE.
				foreach($ads['ad'] as $ad)
				{
					if(!is_numeric($ad['location']))
					{
						continue;
					}

					//SKIP IF NOT HOME PAGE
					if($ad['location'] == 1 && (!is_home() && !(is_front_page() && get_option('page_on_front'))))
					{
						continue;
					}

					//SKIP IF NOT HOME PAGE
					if($ad['location'] == 2 && (is_home() || (is_front_page() && get_option('page_on_front'))))
					{
						continue;
					}

					$ad_html .= '<li>'.preg_replace('/style="[^"]*"/','',$this->$ad_type($ad,0)).'</li>';
					$count++;
				} 
			}
		}

		return $ad_html;
	}

	//STYLES THE SAME WAY AS A WIDGET WOULD.
	function fake_widget($args)
	{
		global $lw_linkads,$lw_options;

		$options = get_option('lw_widget');
		$title_li = $options['title'];

		$links = $this->lw_links_sidebar();

		extract( $args, EXTR_SKIP );

		$content .= $before_widget;
		$content .= "$before_title$title_li$after_title\n\t<ul class='xoxo blogroll'>\n";
		$content .= $links;
		$content .= '</ul>'.$after_widget;

		return $content;
	}

	// ---------------------------------------------------------------------------------------
	// ---------------------------------- AD DISPLAY FUNCTIONS -------------------------------
	// ---------------------------------------------------------------------------------------

	function linkbb($ad,$count=0)
	{
		global $id,$lw_options;

		if(!is_array($ad))
		{
			return '';
		}

		$lw_path = get_option('siteurl');
		$desc = ' title="'.$this->clean_description($ad).'" ';
		$this->link_options($count, $ad);

		if(!$this->do_pretty_links())
		{
			$url = $lw_path.'?'.$lw_options['billboard_base'].'='.$ad['pagename'];
		}
		else
		{
			$url = $lw_path.'/'.$lw_options['billboard_base'].'/'.$ad['pagename'];
		}
		
		$ads =  $lw_options['lw_seperator'].'<a href="'.$url.'"'.$lw_options['lw_style'].$lw_options['rel'].$desc.'>'.$ad['anchor'].'</a>';

		if($ad['link_text'] != '')
		{
			$ads = sprintf($ad['link_text'],$ads);
		}

		return $ads;
	}

	function linkads($ad, $count = 0)
	{
		global $lw_options;
		unset($desc);

		if(!is_array($ad))
		{
			return '';
		}

		$this->link_options($count, $ad);

		if(isset($ad['description']) && !empty($ad['description']))
		{
			$lw_description = ' title="'.$this->clean_description($ad).'" ';
		}

		//SET UP THE LINK PARTS
		$protocol = ($ad['use_https']) ? 'https://' : 'http://';
		$lw_seperator = $lw_options['lw_seperator'];
		$lw_ad_url = $ad['url'];
		$lw_style = $lw_options['lw_style'];
		$lw_rel = $lw_options['rel'];
		$lw_anchor = $ad['anchor'];

		$ads = $lw_seperator.'<a href="'.$protocol.$lw_ad_url.'"'.$lw_style.$lw_rel.$lw_description.'>'.$lw_anchor.'</a>';

		if(!empty($ad['link_text']))
		{
			$ads = sprintf($ad['link_text'],$ads);
		}

		return $ads;
	}

	function linkmura($ad, $count = 0)
	{
		global $lw_options;
		unset($desc);

		if(!is_array($ad))
		{
			return '';
		}

		$this->link_options($count, $ad);

		//NUMBER OF ANCHOR TEXTS
		$lw_arraysize = count($ad['hyperlinks']['hyperlink']);
		//PICK A RANDOM ANCHOR TEXT
		$lw_mura_rand = rand(0,$lw_arraysize-1);
		$ad = $ad['hyperlinks']['hyperlink'][$lw_mura_rand];
		$protocol = ($ad['use_https'])? 'https://' : 'http://';

		if(isset($ad['description']) && !empty($ad['description']))
		{
			$desc = ' title="'.$this->clean_description($ad).'" ';
		}

		$ads =  $lw_options['lw_seperator'].'<a href="'.$protocol.$ad['url'].'"'.$lw_options['lw_style'].$lw_options['rel'].$desc.'>'.$ad['anchor'].'</a>';

		return $ads;
	}

	function linksura($ad, $count = 0)
	{
		global $lw_options;

		if(!is_array($ad))
		{
			return '';
		}

		$this->link_options($count, $ad);

		//NUMBER OF ANCHOR TEXTS
		$lw_arraysize = count($ad['anchors']['anchor']);
		//PICK A RANDOM ANCHOR TEXT
		$lw_sura_rand = rand(0,$lw_arraysize-1);
		$desc = ' title="'.$this->clean_description($ad).'" ';
		$protocol = ($ad['use_https'])? 'https://' : 'http://';
		$ads =  $lw_options['lw_seperator'].'<a href="'.$protocol.$ad['url'].'"'.$lw_options['lw_style'].$lw_options['rel'].$desc.'>'.$ad['anchors']['anchor'][$lw_sura_rand].'</a>';

		if($ad['link_text'] != '')
		{
			$ads = sprintf($ad['link_text'],$ads);
		}

		return $ads;
	}

	//SETS STYLE, REL AND SEPERATOR FOR EACH LINK.
	//USES GLOBAL $LW_OPTIONS ARRAY TO TEMPORARILY STORE INFORMATION.
	function link_options($count, $ad)
	{
		global $lw_options;

		if($lw_options['lw_linkcolor'])
		{
			list($css_color, $css_size) = $this->cloudcss($lw_options['lw_linkcolor']);
			$lw_options['lw_style'] = ' style="color:'.$css_color.'; font-size: '.$css_size.'px;"';
		}
		else
		{
			$lw_options['lw_style'] = '';
		}

		if($ad['nofollow'])
		{
			$lw_options['rel'] = ' rel="nofollow"';
		}
		else
		{
			$lw_options['rel'] = '';
		}
	
		//SORTING OUT WHAT KIND OF LINK SEPARATOR WE NEED TO PRINT, IF ANY
		if($count == 0)
		{
			$lw_options['lw_seperator'] = '';
		}
		elseif(!$lw_options['lw_murasidebar'])
		{
			if(isset($css_color) && !empty($css_color))
			{
				$lw_options['lw_seperator'] = ' &nbsp; &nbsp; &nbsp;';
			}
			else
			{
				$lw_options['lw_seperator'] = ' - ';
			}
		}
	}
	
	// ---------------------------------------------------------------------------------------
	// ---------------------------------- LINKINTEXT FUNCTIONS -------------------------------
	// ---------------------------------------------------------------------------------------

	function lw_linkintxt_dbcleanup()
	{
		//REMOVE OLD LINKINTXT ADVERTS
		global $table_prefix, $wpdb, $user_level;
		$table_name = $table_prefix . "linkintxt";

		$lw_cleandb = "DROP TABLE IF EXISTS ".$table_name;
		@$wpdb->query($lw_cleandb);
	}

	function replace_ok($fwords, $rwords, $content)
	{
		$rcontent = "";
		$ncon = preg_split("/\b".$fwords[0]."\b/i", $content, 2);

		$f = array_shift($fwords);
		$r_lw = array_shift($rwords);

		$lw_oneword = 0;

		for($lw_i = 0; $lw_i < count($ncon); $lw_i++)
		{
			$con = $ncon[$lw_i];

			if(count($fwords))
			{
				$con = $this->replace_ok($fwords, $rwords, $con);
			} 

			if($lw_i < count($ncon)-1)
			{
				$rcontent .= $con.$r_lw;
			}
			else
			{
				$rcontent .= $con;
			}
		}

		return $rcontent;
	}

	//REPLACEONE TAKES THE INPUT WORD AND REPLACES IT WITH THE OUTPUT.
	function ReplaceOne($in, $out, $content)
	{
		if(strpos($content, $in) !== false)
		{
			$pos = strpos($content, $in);
			return substr($content, 0, $pos) . $out . substr($content, $pos+strlen($in));
		}
		else
		{
			return $content;
		}
	}

	//$finds -- AN ARRAY OF WORDS TO LOOK FOR AND REPLACE WITH.
	function lw_contentreplace($content)
	{
		global $user_level, $post,$id;

		if(get_post_meta($id,'_disable_linkworth',true))
		{
			return $content;
		}

		$find_words = array();
		$rep_words = array();

		$finds = get_option('lw_linkintxs');

		if(count($finds) <= 1)
		{
			return $content;
		}

		$links = array();
		$tags = array();

		if(count($finds))
		{
			//STRIP ALL THE EXISTING LINKS OUT FIRST .
			$pattern = '/(<(?:[^<>]+(?:"[^"]*"|\\\'[^\']*\\\')?)+>)/';
			$html_array = preg_split ($pattern, $content, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
			$inlink = false;
			$content = "";
	
			for($lw_i = 0; $lw_i < count($html_array); $lw_i++)
			{
				$chunk = $html_array[$lw_i];

				if((strpos($chunk, "<a") == 0 && strpos($chunk, "<a") !== false) || (strpos($chunk, "<A") == 0 && strpos($chunk, "<A") !== false))
				{
					//STRIP OUT ALL THREE LINK PARTS AND REPLACE WITH A ***
					$links[] = $chunk.$html_array[++$lw_i].$html_array[++$lw_i];
					$content .= "!@#$%^&*()";
				}
				//IF THE FIRST CHARACTER IS AN <
				elseif((strpos($chunk, "<") == 0 && strpos($chunk, "<") !== false))
				{
					//THIS MEANS IT IS A RANDOM TAG, SO STRIP IT AND HOLD IT
					$tags[] = $chunk;
					$content .= "!@#T%^&*()";
				}
				else
				{
					$content .= $chunk;
				}
			}

//			Page matching for linkwords has been removed.

// 			$lw_pageurl = $_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"];
// 			$lw_pageurl = preg_replace('@http(s)*:\/\/(www.)*@','',$lw_pageurl);
// 			$lw_pageurl = rtrim($lw_pageurl,'/');

			foreach($finds as $find)
			{
//				$lw_dispurl = preg_replace('@http(s)*:\/\/(www.)*@','',$find['disp_url']);
//				$lw_dispurl = rtrim($lw_dispurl,'/');
				$this->debug('LinkinTXT: Find:'.$find['find_word'].'; Replace With: '.$find['replace_word']);
				// BELOW LINE SET TO != FOR TESTING PURPOSES
//				if($lw_dispurl = $lw_pageurl)
//				{
					$find_words[] = $find['find_word'];
					$repl_words[] = $find['replace_word'];
//				}
			}
		}

		if(count($find_words))
		{
			$content = $this->replace_ok($find_words, $repl_words, $content);
		}

		foreach($tags as $tag)
		{
			$content = $this->ReplaceOne("!@#T%^&*()", $tag, $content);
		}
	
		foreach($links as $link)
		{
			$content = $this->ReplaceOne("!@#$%^&*()", $link, $content);
		}

		return $content;
	}

	//DELETES SIDEBAR LINKS. ONLY USED FOR UPGRADS FROM VERSION < 309
	function delete_sidebar_links($only_links=0)
	{
		global $wpdb;
		delete_option('lw_links',array());
		//GET THE STORED LINK CATEGORY ID.
		$lw_linkcatid = get_option('lw_linkcatid');
//		update_option('lw_home_links',array());
		if(!function_exists('wp_delete_link'))
		{
			@include_once(ABSPATH.'/wp-admin/includes/bookmark.php');
		}

		if(!$lw_linkcatid)
		{
			return;
		}

		//GET_BOOKMARKS ACCEPTS 1 ARRAY AS AN ARGUMENT.  WE ONLY NEED THE LINKS FOR THE DEFAULT LW CATEGORY SO WE CAN DELETE THEM.
		$args = array();
		//SET UP THE ARRAY ARGUMENTS.
		$args['category'] = $lw_linkcatid;
		$args['hide_invisible'] = 0;
		//CACHE BUSTER
		$args['limit'] = rand(999,9999);
		$links = get_bookmarks($args);

		if(!function_exists('wp_delete_link'))
		{
			@include_once(ABSPATH.'/wp-admin/includes/bookmark.php');
		}

		if(count($links) > 0)
		{
			foreach($links as $link)
			{
				$link_id = $link->link_id;

				do_action( 'delete_link', $link_id );
				wp_delete_object_term_relationships( $link_id, 'link_category' );

				$wpdb->query( $wpdb->prepare( "DELETE FROM $wpdb->links WHERE link_id = %d", $link_id ) );

				do_action( 'deleted_link', $link_id );
				clean_bookmark_cache( $link_id );
			}
		}

		if(!$only_links)
		{
			$args = array();
			//SET UP THE ARRAY ARGUMENTS
			$args['category'] = $lw_linkcatid;
			//CACHE BUSTER
			$args['limit'] = rand(999,9999);
			$links = get_bookmarks($args);
			if(!count($links))
			{
				delete_option('lw_linkcatid');
				//DELETE THE LW CATEGORY.
				wp_delete_term($lw_linkcatid, 'link_category');
			}
			else
			{
				$this->delete_sidebar_links();
			}
		}
	}

	//DEBUGGING INFORMATION - STORES AND PRINTS AN ARRAY.
	function debug($message='')
	{
		static $messages = array();

		$ops = get_option('lw_options');

		if($message)
		{
			$messages[] = $message;
		}
		elseif($ops['debug'] == 1)
		{
			foreach($messages as $debug)
			{
				echo "\n\r".'<!-- LW Debug: '.$debug.' -->';
			}
		}
	}

	//DEACTIVATE THE PLUGIN. DELETES THE OLD LINKS ETC.
	function deactivate_hook()
	{
		$this->delete_sidebar_links();
	}

	//RETURNS A "NORMAL" DESCRIPTION OF THE AD_LOCATION VALUE.
	function human_location($loc)
	{
		if($loc == 1)
		{
			$lw_humanloc = 'Home Page';
		}
		elseif($loc == 2)
		{
			$lw_humanloc = 'Sub Pages';
		}
		elseif($loc == 3)
		{
			$lw_humanloc = 'All Pages';
		}
		else
		{
			$lw_humanloc =$ad['location'];
		}

		return $lw_humanloc;
	}
}//CLASS END

	// -------------------------------------------------------------------------------
	// ------------------------- Initialize the LW CLass -----------------------------
	// -------------------------------------------------------------------------------

$lw_options = get_option('lw_options');

//IF THIS IS MU SITE
if(function_exists('is_site_admin') && function_exists('get_blog_count') && !$lw_options['user_managed'])
{
	include('lw_includes/LinkWorth_MU.php');
}

$LinkWorth = new LinkWorth;
// INITIATE LW: PASSED BY REFERENCE FOR PHP4
add_action('init', array(&$LinkWorth, 'init'));
register_deactivation_hook(__file__,array($LinkWorth,'deactivate_hook'));
//BILLBOARDS
add_filter('generate_rewrite_rules', array(&$LinkWorth,'rewrite'));

//IF WIDGETS ARE SELECTED.
if($lw_options['lw_sidebarwidget'])
{
	add_action('init', 'widget_linkw_init');
}

	// ----------------------------------------------------------------------------
	// ---------------------------------- LW WIDGET -------------------------------
	// ----------------------------------------------------------------------------

function widget_linkw_init()
{
	if(!function_exists('register_sidebar_widget'))
	{
		return;
	}

	function widget_linkw($args)
	{
		global $LinkWorth;

		echo $LinkWorth->fake_widget($args);
	}

	function widget_linkw_control()
	{
		$options = get_option('lw_widget');

		if(!is_array($options))
		{
			$options = array('title'=>'LinkWorth');
			add_option('lw_widget', $options);
		}

		if($_POST['linkw-submit'])
		{
			$options['title'] = strip_tags(stripslashes($_POST['linkw-title']));
			update_option('lw_widget', $options);
		}

		if(!empty($options['title']))
		{
			$title = htmlspecialchars($options['title'], ENT_QUOTES);
		}
		else
		{
			$title = '';
		}
		//HERE IS OUR LITTLE FORM SEGMENT. NOTICE THAT WE DON'T NEED A
		//COMPLETE FORM. THIS WILL BE EMBEDDED INTO THE EXISTING FORM.
?>
		<p style="text-align:right; white-space:nowrap;">
			<label for="linkw-title">Title: <input style="width: 200px;" id="linkw-title" name="linkw-title" type="text" value="<?php echo $title ?>" /></label>
		</p>
		<input type="hidden" id="linkw-submit" name="linkw-submit" value="1" />
<?php
	}

	//THIS REGISTERS OUR WIDGET SO IT APPEARS WITH THE OTHER AVAILABLE
	//WIDGETS AND CAN BE DRAGGED AND DROPPED INTO ANY ACTIVE SIDEBARS.
	wp_register_sidebar_widget('lw_widget','Links Widget','widget_linkw',array('description' => 'LinkWorth widget for publishing deals.'));

	//THIS REGISTERS OUR OPTIONAL WIDGET CONTROL FORM. BECAUSE OF THIS
	//OUR WIDGET WILL HAVE A BUTTON THAT REVEALS A 300X100 PIXEL FORM.
	wp_register_widget_control('lw_widget','Links Widget','widget_linkw_control');
}

	// ---------------------------------------------------------------------------------
	// ---------------------------------- LW ADMIN Panel -------------------------------
	// ---------------------------------------------------------------------------------

//CALLED BY THE ADMIN PANEL FUNCTION TO DISPLAY ADVERTISEMENT INFORMATION. IN ITS OWN FUNCTION TO "KEEP IT CLEAN"
function lw_configpage()
{
	include_once('lw_includes/LinkWorth_admin.php');
}
?>